
/******************************************************************************
* MODULE     : initialize.cc
* DESCRIPTION: Initialize the standard routines
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <evaluator.hh>
#include <function.hh>
#include <single.hh>
#include <printer.hh>
#include <keyword.hh>
namespace mmx {

/******************************************************************************
* All keywords (multiple versions due to continuations)
* should be in same file as where the evaluator_rep constructor
******************************************************************************/

keyword KW_COMPOUND_1 ("compound!", &evaluator_rep::evaluate_compound_1);
keyword KW_COMPOUND_2 ("compound!2", &evaluator_rep::evaluate_compound_2);
keyword KW_COMPOUND_3 ("compound!3", &evaluator_rep::evaluate_compound_3);
keyword KW_EVAL_ARGS_1 ("eval_args!", &evaluator_rep::evaluate_eval_args_1);
keyword KW_LOAD_1 ("load", &evaluator_rep::evaluate_load_1);
keyword KW_LOAD_2 ("load!2", &evaluator_rep::evaluate_load_2);

keyword KW_CONV_APPLY_1 (CONV_APPLY, &evaluator_rep::evaluate_conv_apply_1);
keyword KW_CONV_APPLY_2 ("apply#2", &evaluator_rep::evaluate_conv_apply_2);
keyword KW_CONV_APPLY_3 ("apply#3", &evaluator_rep::evaluate_conv_apply_3);
keyword KW_CONV_CROSS_1 (CONV_CROSS, &evaluator_rep::evaluate_conv_cross_1);
keyword KW_CONV_CROSS_2 ("cross#2", &evaluator_rep::evaluate_conv_cross_2);
keyword KW_CONV_LAMBDA_1 (CONV_LAMBDA, &evaluator_rep::evaluate_conv_lambda_1);
keyword KW_CONV_UNALIAS_1 (CONV_UNALIAS,
  &evaluator_rep::evaluate_conv_unalias_1);
keyword KW_CONV_UNALIAS_2 ("unalias#2",
  &evaluator_rep::evaluate_conv_unalias_2);
keyword KW_CONV_UNAND_1 (CONV_UNAND, &evaluator_rep::evaluate_conv_unand_1);
keyword KW_CONV_UNAND_2 ("unand#2", &evaluator_rep::evaluate_conv_unand_2);
keyword KW_CONV_UNCROSS_1 (CONV_UNCROSS,
  &evaluator_rep::evaluate_conv_uncross_1);
keyword KW_CONV_UNCROSS_2 ("uncross#2",
  &evaluator_rep::evaluate_conv_uncross_2);

keyword KW_ASSIGN_1 (MMX_ASSIGN, &evaluator_rep::evaluate_assign_1);
keyword KW_ASSIGN_2 (":=2", &evaluator_rep::evaluate_assign_2);
keyword KW_ASSIGN_3 (":=3", &evaluator_rep::evaluate_assign_3);
keyword KW_BEGIN_1 (MMX_BEGIN, &evaluator_rep::evaluate_begin_1);
keyword KW_BEGIN_2 ("begin!2", &evaluator_rep::evaluate_begin_2);
keyword KW_CASE_1 (MMX_CASE, &evaluator_rep::evaluate_case_1);
keyword KW_CASE_2 ("case!2", &evaluator_rep::evaluate_case_2);
keyword KW_CASE_3 ("case!3", &evaluator_rep::evaluate_case_3);
keyword KW_CASE_4 ("case!4", &evaluator_rep::evaluate_case_4);
keyword KW_DEFINEALIAS_1 (MMX_DEFINEALIAS, &evaluator_rep::evaluate_define_1);
keyword KW_DEFINE_1 (MMX_DEFINE, &evaluator_rep::evaluate_define_1);
keyword KW_DEFINE_2 ("==2", &evaluator_rep::evaluate_define_2);
keyword KW_IF_1 (MMX_IF, &evaluator_rep::evaluate_if_1);
keyword KW_IF_2 ("if!2", &evaluator_rep::evaluate_if_2);
keyword KW_LAMBDA_1 (MMX_LAMBDA, &evaluator_rep::evaluate_lambda_1);
keyword KW_LOOP_1 (MMX_LOOP, &evaluator_rep::evaluate_loop_1);
keyword KW_LOOP_2 ("loop!2", &evaluator_rep::evaluate_loop_2);
keyword KW_LOOP_3 ("loop!3", &evaluator_rep::evaluate_loop_3);
keyword KW_LOOP_4 ("loop!4", &evaluator_rep::evaluate_loop_4);
keyword KW_LOOP_WHILE ("loop!while", &evaluator_rep::evaluate_loop_while);
keyword KW_LOOP_FOR ("loop!for", &evaluator_rep::evaluate_loop_for);
keyword KW_LOOP_UNTIL ("loop!until", &evaluator_rep::evaluate_loop_until);
keyword KW_LOOP_STEP ("loop!step", &evaluator_rep::evaluate_loop_step);
keyword KW_TRANSTYPE_1 (MMX_TRANSTYPE, &evaluator_rep::evaluate_transtype_1);
keyword KW_TRANSTYPE_2 (":>2", &evaluator_rep::evaluate_transtype_2);

/******************************************************************************
* Some fancy test routines
******************************************************************************/

generic
test_print (generic g) {
  (void) g;
  cout << "Test\n";
  return generic ();
}

static int test_count= 10;

generic
test_bool (generic g) {
  (void) g;
  test_count--;
  return single (test_count >= 0);
}

/******************************************************************************
* Initialization
******************************************************************************/

void
evaluator_rep::declare (keyword kw) {
  env (kw->name)= kw;
}

evaluator_rep::evaluator_rep (): env (table ()) {
  initialize_master ();
  list types   = get_extern_types ();
  list bindings= get_extern_bindings ();
  (void) types;
  while (!nil (bindings)) {
    generic binding= car (bindings);
    declare (as_symbol (binding[0]), value (binding[1], binding[2]));
    bindings= cdr (bindings);
  }

  function test1 ("test_print", test_print);
  function test2 ("test_bool", test_bool);
  declare ("test_print", value (test1, function_type (MMX_VOID)));
  declare ("test_bool", value (test2, function_type (MMX_BOOL)));

  declare ("true", value (single (true), MMX_BOOL));
  declare ("false", value (single (false), MMX_BOOL));
  declare ("mmout", value (printer (symbol ("mmout"), cout), MMX_OSTREAM));
  declare ("mmerr", value (printer (symbol ("mmerr"), cerr), MMX_OSTREAM));

  declare (MMX_GENERIC, value (MMX_GENERIC, MMX_CLASS));
  declare (MMX_VOID, value (MMX_VOID, MMX_CLASS));
  declare (MMX_BOOL, value (MMX_BOOL, MMX_CLASS));
  declare (MMX_INT, value (MMX_INT, MMX_CLASS));
  declare (MMX_STRING, value (MMX_STRING, MMX_CLASS));
  declare (MMX_OSTREAM, value (MMX_OSTREAM, MMX_CLASS));

  declare (KW_COMPOUND_1);
  declare (KW_COMPOUND_2);
  declare (KW_COMPOUND_3);
  declare (KW_EVAL_ARGS_1);
  declare (KW_LOAD_1);
  declare (KW_LOAD_2);

  declare (KW_CONV_APPLY_1);
  declare (KW_CONV_APPLY_2);
  declare (KW_CONV_APPLY_3);
  declare (KW_CONV_CROSS_1);
  declare (KW_CONV_CROSS_2);
  declare (KW_CONV_LAMBDA_1);
  declare (KW_CONV_UNALIAS_1);
  declare (KW_CONV_UNALIAS_2);
  declare (KW_CONV_UNAND_1);
  declare (KW_CONV_UNAND_2);
  declare (KW_CONV_UNCROSS_1);
  declare (KW_CONV_UNCROSS_2);

  declare (KW_ASSIGN_1);
  declare (KW_ASSIGN_2);
  declare (KW_ASSIGN_3);
  declare (KW_BEGIN_1);
  declare (KW_BEGIN_2);
  declare (KW_CASE_1);
  declare (KW_CASE_2);
  declare (KW_CASE_3);
  declare (KW_CASE_4);
  declare (KW_DEFINEALIAS_1);
  declare (KW_DEFINE_1);
  declare (KW_DEFINE_2);
  declare (KW_IF_1);
  declare (KW_IF_2);
  declare (KW_LAMBDA_1);
  declare (KW_LOOP_1);
  declare (KW_LOOP_2);
  declare (KW_LOOP_3);
  declare (KW_LOOP_4);
  declare (KW_LOOP_WHILE);
  declare (KW_LOOP_FOR);
  declare (KW_LOOP_UNTIL);
  declare (KW_LOOP_STEP);
  declare (KW_TRANSTYPE_1);
  declare (KW_TRANSTYPE_2);
}

} // namespace mmx
