
/******************************************************************************
* MODULE     : keyw_define.cc
* DESCRIPTION: Definition of interactive constants
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <evaluator.hh>
namespace mmx {

list
flatten_types (list l) {
  if (nil (l)) return l;
  if (is_func (car (l), MMX_TYPE, 2))
    return cons (car (l) [2], flatten_types (cdr (l)));
  return cons (car (l), flatten_types (cdr (l)));
}

void
evaluator_rep::evaluate_define_1 () {
  // INPUT: (define_1, cont, ...)
  generic decl= car (trace);
  if (!is_func (decl[1], MMX_TYPE, 2)) {
    error (decl, "Bad definition");
    return;
  }

  // transform a declaration like
  //   f(...)...(...):R == ... or
  //   f(...)...(...):R := ...
  // into a declaration var:type ==/:= val first
  generic val = decl[2];
  generic var = decl[1][1];
  generic type= decl[1][2];
  while (is_list (var)) {
    generic dom= cons (MMX_CROSS, flatten_types (cdr (as_list (var))));
    val = list (MMX_LAMBDA, cdr (as_list (var)), type, val);
    type= list (MMX_FUNCTION, dom, type);
    var = var[0];
  }

  if (!is_symbol (var)) {
    error (var, "Not a symbol");
    return;
  }

  cur= list (MMX_TRANSTYPE, val, value (type, MMX_CLASS));
  // FIXME: we should really evaluate type,
  // i.e. replace value (type, MMX_CLASS) by type
  rem= cons (KW_DEFINE_2, var, type, rem);
  // OUTPUT: (body_decl, define_2, var, type, cont, ...)
}

void
evaluator_rep::evaluate_define_2 () {
  // INPUT: (define_2, val, var, type, cont, ...)
  generic decl= car (trace);
  generic val = as_value (car (rem)) -> val;
  generic var = cadr (rem);
  generic type= caddr (rem);

  if (is_func (decl, MMX_DEFINEALIAS, 2)) {
    val = list (val);
    type= list (MMX_ALIAS, type);
  }

  declare (as_symbol (var), value (val, type));
  cur= cadddr (rem);
  rem= cons (VAL_VOID, cddddr (rem));
  // OUTPUT: (cont, void, ...)
  trace= cdr (trace);
}

} // namespace mmx
