
/******************************************************************************
* MODULE     : keyw_loop.cc
* DESCRIPTION: loops
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <evaluator.hh>
#include <single.hh>
namespace mmx {

void
evaluator_rep::evaluate_loop_1 () {
  cur= KW_LOOP_2;
  rem= cons (car(trace)[1], single (true), rem);
  evaluate_loop_2 ();
}

void
evaluator_rep::evaluate_loop_2 () { // pre-loop conditions
  // INPUT: (loop_2, cond_list, first_pass?, ...)
  list l= as_list (car(rem));
  while (!nil (l)) {
    generic tag= car (l);
    if (is_func (tag, MMX_WHILE, 1)) {
      cur= list (MMX_TRANSTYPE, tag[1], value (MMX_BOOL, MMX_CLASS));
      rem= cons (KW_LOOP_WHILE, cdr (l), cdr (rem));
      // OUTPUT: (cond, loop_while, other_conds, first_pass?, ...)
      return;
    }
    else if (is_func (tag, MMX_FOR, 1) && as_bool (cadr (rem))) {
      cur= tag[1];
      rem= cons (KW_LOOP_FOR, cdr (l), cdr (rem));
      // OUTPUT: (instr, loop_for, other_conds, first_pass?, ...)
      return;
    }
    l= cdr (l);
  }
  cur= car(trace)[2];
  rem= cons (KW_LOOP_3, cddr (rem));
  // OUTPUT: (loop_body, loop_3, ...)
}

void
evaluator_rep::evaluate_loop_while () {
  // INPUT: (loop_while, cond, other_conds, first_pass?, cont, ...)
  if (as_bool (as_value (car (rem)) -> val)) {
    cur= KW_LOOP_2;
    rem= cdr (rem);
    // OUTPUT: (loop_2, cond_list, first_pass?, cont, ...)
  }
  else {
    cur= cadddr (rem);
    rem= cons (VAL_VOID, cddddr (rem));
    // OUTPUT: (cont, void, ...)
    trace= cdr (trace);
  }
}

void
evaluator_rep::evaluate_loop_for () {
  // INPUT: (loop_for, result, other_conds, first_pass?, cont, ...)
  cur= KW_LOOP_2;
  rem= cdr (rem);
  // OUTPUT: (loop_2, cond_list, first_pass?, cont, ...)
  evaluate_loop_2 ();
}

void
evaluator_rep::evaluate_loop_3 () { // loop body
  // INPUT: (loop_3, loop_body_result, cont, ...)
  cur= KW_LOOP_4;
  rem= cons (car(trace)[1], cdr (rem));
  // OUTPUT: (loop_4, cond_list, cont, ...)
  evaluate_loop_4 ();
}  

void
evaluator_rep::evaluate_loop_4 () { // post-loop conditions
  // INPUT: (loop_4, cond_list, ...)
  list l= as_list (car (rem));
  while (!nil (l)) {
    generic tag= car (l);
    if (is_func (tag, MMX_UNTIL, 1)) {
      cur= list (MMX_TRANSTYPE, tag[1], value (MMX_BOOL, MMX_CLASS));
      rem= cons (KW_LOOP_UNTIL, cdr (l), cdr (rem));
      // OUTPUT: (cond, loop_until, other_conds, ...)
      return;
    }
    else if (is_func (tag, MMX_STEP, 1)) {
      cur= tag[1];
      rem= cons (KW_LOOP_STEP, cdr (l), cdr (rem));
      // OUTPUT: (instr, loop_step, other_conds, first_pass?, ...)
      return;
    }
    l= cdr (l);
  }
  cur= KW_LOOP_2;
  rem= cons (car(trace)[1], single (false), cdr (rem));
  // OUTPUT: (loop_2, cond_list, false, ...)
  evaluate_loop_2 ();
}

void
evaluator_rep::evaluate_loop_until () {
  // INPUT: (loop_until, cond, other_conds, cont, ...)
  if (as_bool (as_value (car (rem)) -> val)) {
    cur= caddr (rem);
    rem= cons (VAL_VOID, cdddr (rem));
    // OUTPUT: (cont, void, ...)
    trace= cdr (trace);
  }
  else {
    cur= KW_LOOP_4;
    rem= cdr (rem);
    // OUTPUT: (loop_4, cond_list, cont, ...)
  }
}

void
evaluator_rep::evaluate_loop_step () {
  // INPUT: (loop_step, result, other_conds, cont, ...)
  cur= KW_LOOP_4;
  rem= cdr (rem);
  // OUTPUT: (loop_4, cond_list, cont, ...)
  evaluate_loop_4 ();
}

} // namespace mmx
