
/******************************************************************************
* MODULE     : value.cc
* DESCRIPTION: Mathemagix values with a unique type
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <token_symbols.hh>
#include <function.hh>
#include <value.hh>
namespace mmx {

/******************************************************************************
* Routines for the generic type
******************************************************************************/

int
value_rep::Type () {
  return VALUE_TYPE;
}

bool
value_rep::Equal (generic g) {
  if (gentype (g) != VALUE_TYPE) return false;
  return value (this) == as_value (g);
}

generic
value_rep::Copy () {
  return copy (value (this));
}

void
value_rep::Print (ostream& out) {
  out << value (this);
}

ostream&
operator << (ostream& out, value val) {
  if (is_func (val->val, MMX_LAMBDA)) {
    list l= as_list (val->val);
    out << "value<" << cons (car (l), cddr (l)) << "," << val->type << ">";
  }
  else out << "value<" << val->val << "," << val->type << ">";
  return out;
}

value
copy (value val) {
  return value (val->val, val->type);
}

/******************************************************************************
* Overloading and applying a function value to arguments
******************************************************************************/

value VAL_VOID= value (generic (), MMX_VOID);

value
overload (value v1, value v2) {
  if (!is_func (v1->type, MMX_AND))
    return value (list (MMX_AND, v2->val, v1->val),
		  list (MMX_AND, v2->type, v1->type));
  else
    return value (cons (MMX_AND, v2->val, cdr (as_list (v1->val))),
		  cons (MMX_AND, v2->type, cdr (as_list (v1->type))));
}

} // namespace mmx
