
/******************************************************************************
* MODULE     : base_types.cc
* DESCRIPTION: Basic types for the Mathemagix interpreter
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <extern.hh>
#include <single.hh>
#include <printer.hh>
namespace mmx {

/******************************************************************************
* Booleans
******************************************************************************/

generic
bool_not (generic g) {
  return single (!as_bool (g[1]));
}

generic
bool_or (generic g) {
  return single (as_bool (g[1]) | as_bool (g[2]));
}

generic
bool_xor (generic g) {
  return single (as_bool (g[1]) ^ as_bool (g[2]));
}

generic
bool_and (generic g) {
  return single (as_bool (g[1]) & as_bool (g[2]));
}

generic
bool_implies (generic g) {
  return single (as_bool (g[2]) | (!as_bool (g[2])));
}

generic
bool_equivalent (generic g) {
  return single (as_bool (g[1]) == as_bool (g[2]));
}

generic
bool_equal (generic g) {
  return single (as_bool (g[1]) == as_bool (g[2]));
}

generic
bool_not_equal (generic g) {
  return single (as_bool (g[1]) != as_bool (g[2]));
}

generic
bool_output (generic g) {
  printer mmx_out= as_printer (g[1]);
  ostream& out (mmx_out->out);
  if (as_bool (g[2])) out << "true"; else out << "false";
  return mmx_out;
}

/******************************************************************************
* Integers
******************************************************************************/

generic
int_plus (generic g) {
  return single (as_int (g[1]) + as_int (g[2]));
}

generic
int_minus (generic g) {
  return single (as_int (g[1]) - as_int (g[2]));
}

generic
int_times (generic g) {
  return single (as_int (g[1]) * as_int (g[2]));
}

generic
int_equal (generic g) {
  return single (as_int (g[1]) == as_int (g[2]));
}

generic
int_not_equal (generic g) {
  return single (as_int (g[1]) != as_int (g[2]));
}

generic
int_less (generic g) {
  return single (as_int (g[1]) < as_int (g[2]));
}

generic
int_less_equal (generic g) {
  return single (as_int (g[1]) <= as_int (g[2]));
}

generic
int_greater (generic g) {
  return single (as_int (g[1]) > as_int (g[2]));
}

generic
int_greater_equal (generic g) {
  return single (as_int (g[1]) >= as_int (g[2]));
}

generic
int_output (generic g) {
  printer mmx_out= as_printer (g[1]);
  ostream& out (mmx_out->out);
  out << as_int (g[2]);
  return mmx_out;
}

/******************************************************************************
* Strings
******************************************************************************/

generic
string_concat (generic g) {
  return string (as_string (g[1]) * as_string (g[2]));
}

generic
string_equal (generic g) {
  return single (as_string (g[1]) == as_string (g[2]));
}

generic
string_not_equal (generic g) {
  return single (as_string (g[1]) != as_string (g[2]));
}

generic
string_less (generic g) {
  return single (as_string (g[1]) < as_string (g[2]));
}

generic
string_less_equal (generic g) {
  return single (as_string (g[1]) <= as_string (g[2]));
}

generic
string_greater (generic g) {
  return single (as_string (g[2]) < as_string (g[1]));
}

generic
string_greater_equal (generic g) {
  return single (as_string (g[2]) <= as_string (g[1]));
}

generic
string_output (generic g) {
  printer mmx_out= as_printer (g[1]);
  ostream& out (mmx_out->out);
  string s= as_string (g[2]);
  int i, n= N(s);
  for (i=0; i<n; i++) {
    if ((s[i]=='\\') && (i<n-1)) {
      i++;
      if (s[i]=='b') out << '\b';
      else if (s[i]=='n') out << '\n';
      else if (s[i]=='t') out << '\t';
      else out << '\\' << s[i];
    }
    else out << s[i];
  }
  return mmx_out;
}

/******************************************************************************
* Interface
******************************************************************************/

void
initialize_base_types () {
  generic MMX_BOOL_BOOL= function_type (MMX_BOOL, MMX_BOOL);
  generic MMX_BOOL_BOOL_BOOL= function_type (MMX_BOOL, MMX_BOOL, MMX_BOOL);
  generic MMX_INT_INT_INT= function_type (MMX_INT, MMX_INT, MMX_INT);
  generic MMX_INT_INT_BOOL= function_type (MMX_INT, MMX_INT, MMX_BOOL);
  generic MMX_STRING_STRING_STRING=
    function_type (MMX_STRING, MMX_STRING, MMX_STRING);
  generic MMX_STRING_STRING_BOOL= function_type (MMX_STRING, MMX_STRING, MMX_BOOL);

  declare (MMX_NOT, bool_not, MMX_BOOL_BOOL);
  declare (MMX_OR, bool_or, MMX_BOOL_BOOL_BOOL);
  declare (MMX_XOR, bool_xor, MMX_BOOL_BOOL_BOOL);
  declare (MMX_AND, bool_and, MMX_BOOL_BOOL_BOOL);
  declare (MMX_IMPLIES, bool_implies, MMX_BOOL_BOOL_BOOL);
  declare (MMX_EQUIV, bool_equivalent, MMX_BOOL_BOOL_BOOL);
  declare (MMX_EQUAL, bool_equal, MMX_BOOL_BOOL_BOOL);
  declare (MMX_UNEQUAL, bool_not_equal, MMX_BOOL_BOOL_BOOL);
  declare (MMX_LESSLESS, bool_output,
	   function_type (MMX_OSTREAM, MMX_BOOL, MMX_OSTREAM));

  declare (MMX_PLUS, int_plus, MMX_INT_INT_INT);
  declare (MMX_MINUS, int_minus, MMX_INT_INT_INT);
  declare (MMX_TIMES, int_times, MMX_INT_INT_INT);
  declare (MMX_EQUAL, int_equal, MMX_INT_INT_BOOL);
  declare (MMX_UNEQUAL, int_not_equal, MMX_INT_INT_BOOL);
  declare (MMX_LESS, int_less, MMX_INT_INT_BOOL);
  declare (MMX_LESSEQ, int_less_equal, MMX_INT_INT_BOOL);
  declare (MMX_GTR, int_greater, MMX_INT_INT_BOOL);
  declare (MMX_GTREQ, int_greater_equal, MMX_INT_INT_BOOL);
  declare (MMX_LESSLESS, int_output,
	   function_type (MMX_OSTREAM, MMX_INT, MMX_OSTREAM));

  declare (MMX_TIMES, string_concat, MMX_STRING_STRING_STRING);
  declare (MMX_EQUAL, string_equal, MMX_STRING_STRING_BOOL);
  declare (MMX_UNEQUAL, string_not_equal, MMX_STRING_STRING_BOOL);
  declare (MMX_LESS, string_less, MMX_STRING_STRING_BOOL);
  declare (MMX_LESSEQ, string_less_equal, MMX_STRING_STRING_BOOL);
  declare (MMX_GTR, string_greater, MMX_STRING_STRING_BOOL);
  declare (MMX_GTREQ, string_greater_equal, MMX_STRING_STRING_BOOL);
  declare (MMX_LESSLESS, string_output,
	   function_type (MMX_OSTREAM, MMX_STRING, MMX_OSTREAM));
}

} // namespace mmx
