
/******************************************************************************
* MODULE     : logic_apply.cc
* DESCRIPTION: construct a program for function application + conversion
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <converter.hh>
#include <single.hh>
#include <quantor.hh>
namespace mmx {

/******************************************************************************
* Translation of conversion proof into a conversion program
******************************************************************************/

static generic make_prog (proof p, generic arg, int level);

static symbol
make_var (int nr) {
  return symbol (as_string (nr) * "#");
}

static list
make_prog_cross (list ps, generic arg, int nr, int level) {
  if (nil (ps)) return ps;
  generic subarg= list (CONV_UNCROSS, arg, single (nr));
  return cons (make_prog (as_proof (car (ps)), subarg, level),
	       make_prog_cross (cdr (ps), arg, nr+1, level));
}

static generic
make_prog (proof p, generic arg, int level) {
  // cout << "Make prog " << p << ", " << arg << lf;
  if (p->rule == symbol ("given")) return arg;
  if (p->rule == symbol ("match")) return arg;
  if (p->rule == symbol ("=>arrow")) {
    generic var = make_var (level);
    generic prg1= make_prog (as_proof (p->deps[0]), var , level+1);
    generic prg2= list (CONV_APPLY, arg, prg1);
    generic prg3= make_prog (as_proof (p->deps[1]), prg2, level+1);
    return list (CONV_LAMBDA, var, prg3);
  }
  if (p->rule == symbol ("alias=>")) {
    generic prg= make_prog (as_proof (p->deps[0]), arg, level);
    return list (CONV_UNALIAS, prg);
  }
  if (is_func (p->rule, "and=>", 1)) {
    generic pjt= list (CONV_UNAND, arg, p->rule[1]);
    return make_prog (as_proof (p->deps[0]), pjt, level);
  }
  if (p->rule == symbol ("=>cross")) {
    list l= make_prog_cross (p->deps, arg, 1, level);
    return cons (CONV_CROSS, l);
  }
  fatal_error ("proof cannot yet be treated", "make_prog", "apply.cc");
  return generic ();
}

static generic
make_prog (proof p, generic arg1, generic arg2, int level) {
  // cout << "Make prog " << p << ", " << arg1 << ", " << arg2 << lf;
  if (p->rule == symbol ("apply")) {
    generic prg1= make_prog (as_proof (p->deps[0]), arg1, level);
    generic prg2= make_prog (as_proof (p->deps[1]), arg2, level);
    return list (CONV_APPLY, prg1, prg2);
  }
  fatal_error ("proof cannot yet be treated", "make_prog", "apply.cc");
  return generic ();
}

/******************************************************************************
* Simplification of conversion programs
******************************************************************************/

static generic simplify_prog (generic prg);

static generic
simplify_prog (list l) {
  if (nil (l)) return l;
  return cons (simplify_prog (car (l)), simplify_prog (cdr (l)));
}

static generic
simplify_prog (generic prg) {
  if (is_func (prg, CONV_APPLY, 2) && is_func (prg[1], CONV_LAMBDA, 2))
    return substitute (prg[1][2], prg[1][1], prg[2]);
  if (!is_list (prg)) return prg;
  return simplify_prog (as_list (prg));
}

/******************************************************************************
* Interface
******************************************************************************/

generic
converter_rep::apply_prog (generic f, generic arg) {
  proof q (arg, symbol ("given"), list (), table ());
  proof p= apply (f, q, CV_NORMAL);
  if (nil (p)) return generic ();
  generic prg= make_prog (p, make_var (0), make_var (1), 2);
  return list (simplify_prog (prg), p->f);
}

} // namespace mmx
