
/******************************************************************************
* MODULE     : list.hh
* DESCRIPTION: lists
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __LIST_HH
#define __LIST_HH
#include <generic.hh>
namespace mmx {

class list;
class list_rep: generic_rep {
protected:
  generic item;
  list_rep* next;

  int Type ();
  int Hash ();
  bool Equal (generic x);
  generic Copy ();
  int Arity ();
  generic& Access (int i);
  generic Range (int begin, int end);
  generic& Car ();
  generic Cdr ();
  void Print (ostream& out);

public:
  list_rep (generic g);
  list_rep (generic g, list l);

  friend class list;
  friend generic& car (list l);
  friend generic& cadr (list l);
  friend generic& caddr (list l);
  friend generic& cadddr (list l);
  friend list&    cdr (list l);
  friend list&    cddr (list l);
  friend list&    cdddr (list l);
  friend list&    cddddr (list l);
  friend bool     atom (list l);
};

class list {
protected:
  list_rep* rep;
  inline list (list_rep* rep2): rep (rep2) {}
  // inline list (generic g1, list l): rep (new list_rep (g1, l)) {}

public:
  inline list (): rep (NULL) {}
  inline list (generic g1): rep (new list_rep (g1)) {}
  inline list (generic g1, generic g2): rep (new list_rep (g1, list (g2))) {}
  inline list (generic g1, generic g2, generic g3):
    rep (new list_rep (g1, list (g2, g3))) {}
  inline list (generic g1, generic g2, generic g3, generic g4):
    rep (new list_rep (g1, list (g2, g3, g4))) {}
  inline list (generic g1, generic g2, generic g3, generic g4, generic g5):
    rep (new list_rep (g1, list (g2, g3, g4, g5))) {}
  inline friend list as_list (generic g) {
    if (nil (g)) return list ();
    if (gentype(g) != LIST_TYPE) type_mismatch (g, "list");
    return list ((list_rep*) g.rep); }
  inline operator generic () { return generic (rep); }
  inline list_rep* operator -> () { return rep; }

  friend int N (list l);
  friend bool operator == (list a, list b);
  friend bool operator != (list a, list b);
  friend list copy (list l);
  generic& operator [] (int i);
  list operator () (int start, int end);
  inline friend bool nil (list l) { return l.rep == NULL; }
  inline friend bool atom (list l) {
    return (l.rep != NULL) && (l.rep->next == NULL); }
  friend ostream& operator << (ostream& out, list a);

  inline friend generic& car (list l) {
    return l.rep->item; }
  inline friend generic& cadr (list l) {
    return l.rep->next->item; }
  inline friend generic& caddr (list l) {
    return l.rep->next->next->item; }
  inline friend generic& cadddr (list l) {
    return l.rep->next->next->next->item; }
  inline friend list& cdr (list l) {
    return *((list*) ((void*) (&l.rep->next))); } // UNSAFE
  inline friend list& cddr (list l) {
    return *((list*) ((void*) (&l.rep->next->next))); } // UNSAFE
  inline friend list& cdddr (list l) {
    return *((list*) ((void*) (&l.rep->next->next->next))); } // UNSAFE
  inline friend list& cddddr (list l) {
    return *((list*) ((void*) (&l.rep->next->next->next->next))); } // UNSAFE


  inline friend list cons (generic g1, list g2) {
    return list (new list_rep (g1, g2)); }
  inline friend list cons (generic g1, generic g2) {
    return cons (g1, as_list (g2)); }
  inline friend list cons (generic g1, generic g2, generic g3) {
    return cons (g1, cons (g2, g3)); }
  inline friend list cons (generic g1, generic g2, generic g3, generic g4) {
    return cons (g1, cons (g2, g3, g4)); }
  inline friend list cons (generic g1, generic g2, generic g3,
			   generic g4, generic g5) {
    return cons (g1, cons (g2, g3, g4, g5)); }

  friend class list_rep;
};

inline generic cAr (list l) { return l[N(l)-1]; }
inline list cDr (list l) { return l(0,N(l)-1); }

list reverse (list l);
list operator * (list l1, list l2);
list apply_all (generic g, list l);
bool is_list (generic g, generic g1);
bool is_list (generic g, generic g1, generic g2);

}
#endif // __LIST_HH
